@@ -2,6 +2,8 @@
 
 # Copyright (C) 2012 Midokura Japan K.K.
 # Copyright (C) 2013 Midokura PTE LTD
+# Copyright (C) 2014 Midokura SARL.
+# Copyright (C) 2015 Midokura SARL.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -15,24 +17,20 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-#
-# @author: Takaaki Suzuki, Midokura Japan KK
-# @author: Tomoe Sugihara, Midokura Japan KK
-# @author: Ryu Ishimoto, Midokura Japan KK
-# @author: Rossella Sblendido, Midokura Japan KK
-# @author: Duarte Nunes, Midokura Japan KK
 
-from webob import exc as w_exc
-
-from midonet.neutron.common import config  # noqa
-from midonet.neutron.db import db_util
-from midonetclient import exc
-from midonetclient.neutron import client as n_client
 
 from oslo.config import cfg
+from oslo.db import exception as db_exc
 
-from sqlalchemy import exc as sa_exc
+from midonet.neutron import api
+from midonet.neutron.common import config  # noqa
+from midonet.neutron.common import util
+from midonet.neutron.db import db_util
+from midonet.neutron import extensions
+from midonetclient import client
 
+from neutron.api import extensions as neutron_extensions
+from neutron.api.rpc.handlers import dhcp_rpc
 from neutron.common import constants as n_const
 from neutron.common import exceptions as n_exc
 from neutron.common import rpc as n_rpc
@@ -40,78 +38,63 @@ from neutron.common import topics
 from neutron.common import utils
 from neutron.db import agents_db
 from neutron.db import agentschedulers_db
-from neutron.db import api as db
 from neutron.db import db_base_plugin_v2
-from neutron.db import dhcp_rpc_base
 from neutron.db import external_net_db
+from neutron.db import extradhcpopt_db
 from neutron.db import l3_gwmode_db
 from neutron.db.loadbalancer import loadbalancer_db
 from neutron.db import portbindings_db
 from neutron.db import routedserviceinsertion_db as rsi_db
 from neutron.db import securitygroups_db
+from neutron.extensions import extra_dhcp_opt as edo_ext
 from neutron.extensions import portbindings
 from neutron.extensions import routedserviceinsertion as rsi
 from neutron.extensions import securitygroup as ext_sg
 from neutron.openstack.common import excutils
 from neutron.openstack.common import importutils
 from neutron.openstack.common import log as logging
-from neutron.openstack.common import rpc
 from neutron.plugins.common import constants
 
+
 LOG = logging.getLogger(__name__)
 
 
-def handle_api_error(fn):
-    """Wrapper for methods that throws custom exceptions."""
-    def wrapped(*args, **kwargs):
-        try:
-            return fn(*args, **kwargs)
-        except (w_exc.HTTPException, exc.MidoApiConnectionError) as ex:
-            raise MidonetApiException(msg=ex)
-    return wrapped
-
-
-class MidonetApiException(n_exc.NeutronException):
-        message = _("MidoNet API error: %(msg)s")
-
-
-class MidoRpcCallbacks(dhcp_rpc_base.DhcpRpcCallbackMixin):
-    RPC_API_VERSION = '1.1'
-
-    def create_rpc_dispatcher(self):
-        """Get the rpc dispatcher for this manager.
-
-        This a basic implementation that will call the plugin like get_ports
-        and handle basic events
-        If a manager would like to set an rpc API version, or support more than
-        one class as the target of rpc messages, override this method.
-        """
-        return n_rpc.PluginRpcDispatcher([self,
-                                          agents_db.AgentExtRpcCallback()])
-
-
-class MidonetPluginException(n_exc.NeutronException):
-    message = _("%(msg)s")
-
-
 class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
                       portbindings_db.PortBindingMixin,
                       external_net_db.External_net_db_mixin,
+                      extradhcpopt_db.ExtraDhcpOptMixin,
                       l3_gwmode_db.L3_NAT_db_mixin,
                       agentschedulers_db.DhcpAgentSchedulerDbMixin,
                       securitygroups_db.SecurityGroupDbMixin,
                       rsi_db.RoutedServiceInsertionDbMixin,
-                      loadbalancer_db.LoadBalancerPluginDb):
+                      loadbalancer_db.LoadBalancerPluginDb,
+                      api.MidoNetApiMixin):
 
     supported_extension_aliases = ['agent',
                                    'binding',
+                                   'bgp',
+                                   'cluster',
+                                   'chain-rule',
                                    'dhcp_agent_scheduler',
                                    'external-net',
+                                   'extra_dhcp_opt',
+                                   'ip-addr-group',
+                                   'license',
+                                   'midonet-subnet',
                                    'router',
+                                   'host',
+                                   'bridge',
+                                   'midonet-port',
+                                   'midonet-router',
+                                   'port-group',
                                    'quotas',
                                    'security-group',
+                                   'system',
                                    'routed-service-insertion',
-                                   'lbaas']
+                                   'routing-table',
+                                   'vtep',
+                                   'lbaas',
+                                   'tunnelzone']
     __native_bulk_support = True
 
     def __init__(self):
@@ -119,15 +102,16 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
 
         # Instantiate MidoNet API client
         conf = cfg.CONF.MIDONET
-        self.api_cli = n_client.MidonetClient(conf.midonet_uri, conf.username,
-                                              conf.password,
-                                              project_id=conf.project_id)
+        neutron_extensions.append_api_extensions_path(extensions.__path__)
+        self.api_cli = client.MidonetClient(conf.midonet_uri, conf.username,
+                                            conf.password,
+                                            project_id=conf.project_id)
 
         self.setup_rpc()
-        self.repair_quotas_table()
 
         self.base_binding_dict = {
             portbindings.VIF_TYPE: portbindings.VIF_TYPE_MIDONET,
+            portbindings.VNIC_TYPE: portbindings.VNIC_NORMAL,
             portbindings.VIF_DETAILS: {
                 # TODO(rkukura): Replace with new VIF security details
                 portbindings.CAP_PORT_FILTER:
@@ -139,23 +123,13 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
     def setup_rpc(self):
         # RPC support
         self.topic = topics.PLUGIN
-        self.conn = rpc.create_connection(new=True)
-        self.callbacks = MidoRpcCallbacks()
-        self.dispatcher = self.callbacks.create_rpc_dispatcher()
-        self.conn.create_consumer(self.topic, self.dispatcher,
+        self.conn = n_rpc.create_connection(new=True)
+        self.endpoints = [dhcp_rpc.DhcpRpcCallback(),
+                          agents_db.AgentExtRpcCallback()]
+        self.conn.create_consumer(self.topic, self.endpoints,
                                   fanout=False)
         # Consume from all consumers in a thread
-        self.conn.consume_in_thread()
-
-    def repair_quotas_table(self):
-        import neutron.db.quota_db as qdb
-        quota_table = qdb.Quota()
-        session = db.get_session()
-        try:
-            quota_table.__table__.create(bind=session.bind)
-        except sa_exc.OperationalError:
-            # If the table already exists, then this is expected.
-            pass
+        self.conn.consume_in_threads()
 
     def _process_create_network(self, context, network):
 
@@ -170,7 +144,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
 
         return net
 
-    @handle_api_error
+    @util.handle_api_error
     def create_network(self, context, network):
         """Create Neutron network.
 
@@ -192,7 +166,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
         LOG.info(_("MidonetPluginV2.create_network exiting: net=%r"), net)
         return net
 
-    @handle_api_error
+    @util.handle_api_error
     def update_network(self, context, id, network):
         """Update Neutron network.
 
@@ -212,10 +186,19 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
         LOG.info(_("MidonetPluginV2.update_network exiting: net=%r"), net)
         return net
 
-    @handle_api_error
+    @util.handle_api_error
     @utils.synchronized('midonet-network-lock', external=True)
+    @util.retry_on_error(2, 1, db_exc.DBError)
     def delete_network(self, context, id):
-        """Delete a network and its corresponding MidoNet bridge."""
+        """Delete a network and its corresponding MidoNet bridge.
+
+        This method is wrapped by 'retry_on_error' decorator because concurrent
+        requests to the API server often causes DB deadlock error because
+        eventlet green threads do not yield properly when they block inside
+        the transaction.  This hack should no longer become available once
+        we moved to the model where API requests are asynchronous or when
+        eventlet-compatible mysqlconnector is used for the DB driver instead.
+        """
         LOG.info(_("MidonetPluginV2.delete_network called: id=%r"), id)
 
         with context.session.begin(subtransactions=True):
@@ -225,7 +208,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
 
         LOG.info(_("MidonetPluginV2.delete_network exiting: id=%r"), id)
 
-    @handle_api_error
+    @util.handle_api_error
     def create_subnet(self, context, subnet):
         """Create Neutron subnet.
 
@@ -248,7 +231,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
                  sn_entry)
         return sn_entry
 
-    @handle_api_error
+    @util.handle_api_error
     def delete_subnet(self, context, id):
         """Delete Neutron subnet.
 
@@ -262,7 +245,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
 
         LOG.info(_("MidonetPluginV2.delete_subnet exiting"))
 
-    @handle_api_error
+    @util.handle_api_error
     def update_subnet(self, context, id, subnet):
         """Update the subnet with new info.
         """
@@ -280,6 +263,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
         with context.session.begin(subtransactions=True):
             # Create a Neutron port
             new_port = super(MidonetPluginV2, self).create_port(context, port)
+            dhcp_opts = port['port'].get(edo_ext.EXTRADHCPOPTS, [])
 
             # Make sure that the port created is valid
             if "id" not in new_port:
@@ -296,10 +280,12 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
 
             self._process_portbindings_create_and_update(context, port_data,
                                                          new_port)
+            self._process_port_create_extra_dhcp_opts(context, new_port,
+                                                      dhcp_opts)
 
         return new_port
 
-    @handle_api_error
+    @util.handle_api_error
     @utils.synchronized('midonet-port-lock', external=True)
     def create_port(self, context, port):
         """Create a L2 port in Neutron/MidoNet."""
@@ -319,8 +305,21 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
         LOG.info(_("MidonetPluginV2.create_port exiting: port=%r"), new_port)
         return new_port
 
-    @handle_api_error
+    @util.handle_api_error
     @utils.synchronized('midonet-port-lock', external=True)
+    @util.retry_on_error(2, 1, db_exc.DBError)
+    def _process_port_delete(self, context, id):
+        """Delete the Neutron and MidoNet ports
+
+        This method is wrapped by 'retry_on_error' decorator.  See the
+        explanation in the 'delete_network' comment.
+        """
+        with context.session.begin(subtransactions=True):
+            super(MidonetPluginV2, self).disassociate_floatingips(
+                context, id, do_notify=False)
+            super(MidonetPluginV2, self).delete_port(context, id)
+            self.api_cli.delete_port(id)
+
     def delete_port(self, context, id, l3_port_check=True):
         """Delete a neutron port and corresponding MidoNet bridge port."""
         LOG.info(_("MidonetPluginV2.delete_port called: id=%(id)s "
@@ -332,17 +331,14 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
         if l3_port_check:
             self.prevent_l3_port_deletion(context, id)
 
-        with context.session.begin(subtransactions=True):
-            super(MidonetPluginV2, self).disassociate_floatingips(context, id)
-            super(MidonetPluginV2, self).delete_port(context, id)
-            self.api_cli.delete_port(id)
-
+        self._process_port_delete(context, id)
         LOG.info(_("MidonetPluginV2.delete_port exiting: id=%r"), id)
 
     def _process_port_update(self, context, id, in_port, out_port):
 
         has_sg = self._check_update_has_security_groups(in_port)
         delete_sg = self._check_update_deletes_security_groups(in_port)
+        self._update_extra_dhcp_opts_on_port(context, id, in_port, out_port)
 
         if delete_sg or has_sg:
             # delete the port binding and read it with the new rules.
@@ -350,7 +346,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
             sg_ids = self._get_security_groups_on_port(context, in_port)
             self._process_port_create_security_group(context, out_port, sg_ids)
 
-    @handle_api_error
+    @util.handle_api_error
     def update_port(self, context, id, port):
         """Handle port update, including security groups and fixed IPs."""
         LOG.info(_("MidonetPluginV2.update_port called: id=%(id)s "
@@ -368,7 +364,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
         LOG.info(_("MidonetPluginV2.update_port exiting: p=%r"), p)
         return p
 
-    @handle_api_error
+    @util.handle_api_error
     def create_router(self, context, router):
         """Handle router creation.
 
@@ -394,7 +390,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
                    "router=%(router)s."), {"router": r})
         return r
 
-    @handle_api_error
+    @util.handle_api_error
     def update_router(self, context, id, router):
         """Handle router updates."""
         LOG.info(_("MidonetPluginV2.update_router called: id=%(id)s "
@@ -407,7 +403,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
         LOG.info(_("MidonetPluginV2.update_router exiting: router=%r"), r)
         return r
 
-    @handle_api_error
+    @util.handle_api_error
     def delete_router(self, context, id):
         """Handler for router deletion.
 
@@ -424,7 +420,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
 
         LOG.info(_("MidonetPluginV2.delete_router exiting: id=%s"), id)
 
-    @handle_api_error
+    @util.handle_api_error
     def add_router_interface(self, context, router_id, interface_info):
         """Handle router linking with network."""
         LOG.info(_("MidonetPluginV2.add_router_interface called: "
@@ -448,7 +444,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
                  info)
         return info
 
-    @handle_api_error
+    @util.handle_api_error
     def remove_router_interface(self, context, router_id, interface_info):
         """Handle router un-linking with network."""
         LOG.info(_("MidonetPluginV2.remove_router_interface called: "
@@ -465,7 +461,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
                    "info=%r"), info)
         return info
 
-    @handle_api_error
+    @util.handle_api_error
     def create_floatingip(self, context, floatingip):
         """Handle floating IP creation."""
         LOG.info(_("MidonetPluginV2.create_floatingip called: ip=%r"),
@@ -486,7 +482,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
                  fip)
         return fip
 
-    @handle_api_error
+    @util.handle_api_error
     def delete_floatingip(self, context, id):
         """Handle floating IP deletion."""
         LOG.info(_("MidonetPluginV2.delete_floatingip called: id=%s"), id)
@@ -497,7 +493,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
 
         LOG.info(_("MidonetPluginV2.delete_floatingip exiting: id=%r"), id)
 
-    @handle_api_error
+    @util.handle_api_error
     def update_floatingip(self, context, id, floatingip):
         """Handle floating IP association and disassociation."""
         LOG.info(_("MidonetPluginV2.update_floatingip called: id=%(id)s "
@@ -507,7 +503,6 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
         with context.session.begin(subtransactions=True):
             fip = super(MidonetPluginV2, self).update_floatingip(context, id,
                                                                  floatingip)
-
             # Update status based on association
             if fip.get('port_id') is None:
                 fip['status'] = n_const.FLOATINGIP_STATUS_DOWN
@@ -520,7 +515,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
         LOG.info(_("MidonetPluginV2.update_floating_ip exiting: fip=%s"), fip)
         return fip
 
-    @handle_api_error
+    @util.handle_api_error
     def create_security_group(self, context, security_group, default_sg=False):
         """Create security group.
 
@@ -555,7 +550,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
         LOG.info(_("MidonetPluginV2.create_security_group exiting: sg=%r"), sg)
         return sg
 
-    @handle_api_error
+    @util.handle_api_error
     def delete_security_group(self, context, id):
         """Delete chains for Neutron security group."""
         LOG.info(_("MidonetPluginV2.delete_security_group called: id=%s"), id)
@@ -573,7 +568,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
 
         LOG.info(_("MidonetPluginV2.delete_security_group exiting: id=%r"), id)
 
-    @handle_api_error
+    @util.handle_api_error
     def create_security_group_rule(self, context, security_group_rule):
         """Create a security group rule
 
@@ -600,7 +595,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
                    "rule=%r"), rule)
         return rule
 
-    @handle_api_error
+    @util.handle_api_error
     def create_security_group_rule_bulk(self, context, security_group_rules):
         """Create multiple security group rules
 
@@ -628,7 +623,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
                    "rules=%r"), rules)
         return rules
 
-    @handle_api_error
+    @util.handle_api_error
     def delete_security_group_rule(self, context, sg_rule_id):
         """Delete a security group rule
 
@@ -662,14 +657,16 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
                         "VIP subnet is external")
                 raise n_exc.BadRequest(resource='router', msg=msg)
 
-    @handle_api_error
+    @util.handle_api_error
     def create_vip(self, context, vip):
         LOG.debug("MidonetPluginV2.create_vip called: %(vip)r",
                   {'vip': vip})
 
         with context.session.begin(subtransactions=True):
+
             self._validate_vip_subnet(context, vip['vip']['subnet_id'],
                                       vip['vip']['pool_id'])
+
             v = super(MidonetPluginV2, self).create_vip(context, vip)
             self.api_cli.create_vip(v)
             v['status'] = constants.ACTIVE
@@ -679,7 +676,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
         LOG.debug("MidonetPluginV2.create_vip exiting: id=%r", v['id'])
         return v
 
-    @handle_api_error
+    @util.handle_api_error
     def delete_vip(self, context, id):
         LOG.debug("MidonetPluginV2.delete_vip called: id=%(id)r",
                   {'id': id})
@@ -691,7 +688,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
         LOG.debug("MidonetPluginV2.delete_vip existing: id=%(id)r",
                   {'id': id})
 
-    @handle_api_error
+    @util.handle_api_error
     def update_vip(self, context, id, vip):
         LOG.debug("MidonetPluginV2.update_vip called: id=%(id)r, "
                   "vip=%(vip)r", {'id': id, 'vip': vip})
@@ -704,7 +701,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
                   "vip=%(vip)r", {'id': id, 'vip': v})
         return v
 
-    @handle_api_error
+    @util.handle_api_error
     def create_pool(self, context, pool):
         LOG.debug("MidonetPluginV2.create_pool called: %(pool)r",
                   {'pool': pool})
@@ -747,7 +744,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
                   {'pool': p})
         return p
 
-    @handle_api_error
+    @util.handle_api_error
     def update_pool(self, context, id, pool):
         LOG.debug("MidonetPluginV2.update_pool called: id=%(id)r, "
                   "pool=%(pool)r", {'id': id, 'pool': pool})
@@ -760,7 +757,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
                   "pool=%(pool)r", {'id': id, 'pool': pool})
         return p
 
-    @handle_api_error
+    @util.handle_api_error
     def delete_pool(self, context, id):
         LOG.debug("MidonetPluginV2.delete_pool called: %(id)r", {'id': id})
 
@@ -772,7 +769,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
 
         LOG.debug("MidonetPluginV2.delete_pool exiting: %(id)r", {'id': id})
 
-    @handle_api_error
+    @util.handle_api_error
     def create_member(self, context, member):
         LOG.debug("MidonetPluginV2.create_member called: %(member)r",
                   {'member': member})
@@ -788,7 +785,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
                   {'member': m})
         return m
 
-    @handle_api_error
+    @util.handle_api_error
     def update_member(self, context, id, member):
         LOG.debug("MidonetPluginV2.update_member called: id=%(id)r, "
                   "member=%(member)r", {'id': id, 'member': member})
@@ -801,7 +798,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
                   "member=%(member)r", {'id': id, 'member': m})
         return m
 
-    @handle_api_error
+    @util.handle_api_error
     def delete_member(self, context, id):
         LOG.debug("MidonetPluginV2.delete_member called: %(id)r",
                   {'id': id})
@@ -813,7 +810,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
         LOG.debug("MidonetPluginV2.delete_member exiting: %(id)r",
                   {'id': id})
 
-    @handle_api_error
+    @util.handle_api_error
     def create_health_monitor(self, context, health_monitor):
         LOG.debug("MidonetPluginV2.create_health_monitor called: "
                   " %(health_monitor)r", {'health_monitor': health_monitor})
@@ -827,7 +824,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
                   "%(health_monitor)r", {'health_monitor': hm})
         return hm
 
-    @handle_api_error
+    @util.handle_api_error
     def update_health_monitor(self, context, id, health_monitor):
         LOG.debug("MidonetPluginV2.update_health_monitor called: id=%(id)r, "
                   "health_monitor=%(health_monitor)r",
@@ -843,7 +840,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
                   {'id': id, 'health_monitor': hm})
         return hm
 
-    @handle_api_error
+    @util.handle_api_error
     def delete_health_monitor(self, context, id):
         LOG.debug("MidonetPluginV2.delete_health_monitor called: %(id)r",
                   {'id': id})
@@ -855,7 +852,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
         LOG.debug("MidonetPluginV2.delete_health_monitor exiting: %(id)r",
                   {'id': id})
 
-    @handle_api_error
+    @util.handle_api_error
     def create_pool_health_monitor(self, context, health_monitor, pool_id):
         LOG.debug("MidonetPluginV2.create_pool_health_monitor called: "
                   "hm=%(health_monitor)r, pool_id=%(pool_id)r",
@@ -878,7 +875,7 @@ class MidonetPluginV2(db_base_plugin_v2.NeutronDbPluginV2,
                   {'health_monitor': health_monitor, 'pool_id': pool_id})
         return monitors
 
-    @handle_api_error
+    @util.handle_api_error
     def delete_pool_health_monitor(self, context, id, pool_id):
         LOG.debug("MidonetPluginV2.delete_pool_health_monitor called: "
                   "id=%(id)r, pool_id=%(pool_id)r",
